﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SunCRC.Crc.Crc1_64
{
    class Crc1_64
    {
        private Crc1_64Model crcModel;
        private ulong[] crcTable;

        public Crc1_64(Crc1_64Model model)
        {
            this.crcModel = model;
            CalculateCrcTable();
        }

        public ulong Compute_Simple(byte[] bytes)
        {
            ulong crc = crcModel.Initial;
            if (crcModel.CrcSize < 8)
            {
                crc <<= crcModel.BitOffsetTo8Bits;
            }

            foreach (byte b in bytes)
            {
                byte curByte = (crcModel.InputReflected ? CrcUtil.Reflect8(b) : b);
                crc ^= (ulong)((ulong)curByte << crcModel.ShiftByteLeftBits); /* move byte into MSB of 64bit CRC */

                for (int i = 0; i < 8; i++)
                {
                    if ((crc & crcModel.TopBitMask) != 0)
                    {
                        crc = (ulong)((crc << 1) ^ crcModel.Polynomial);
                    }
                    else
                    {
                        crc <<= 1;
                    }
                }

                crc = crc & crcModel.FinalMask;
            }

            if (crcModel.CrcSize < 8)
            {
                crc >>= crcModel.BitOffsetTo8Bits;
            }

            crc = (crcModel.ResultReflected ? CrcUtil.ReflectGeneral(crc, crcModel.CrcSize) : crc);
            crc ^= crcModel.FinalXor;

            return crc;
        }

        /// <summary>
        /// Precalculate look up table.
        /// </summary>
        private void CalculateCrcTable()
        {
            crcTable = new ulong[256];

            for (int divident = 0; divident < 256; divident++)
            {
                ulong curByte = (ulong)((ulong)divident << crcModel.ShiftByteLeftBits);
                for (byte bit = 0; bit < 8; bit++)
                {
                    if ((curByte & crcModel.TopBitMask) != 0)
                    {
                        curByte <<= 1;
                        curByte ^= crcModel.Polynomial;
                    }
                    else
                    {
                        curByte <<= 1;
                    }
                }

                crcTable[divident] = curByte & crcModel.FinalMask;
            }
        }

        /// <summary>
        /// Calculate using lookup table.
        /// </summary>
        /// <param name="bytes"></param>
        /// <returns></returns>
        public ulong Compute(byte[] bytes)
        {
            ulong crc = crcModel.Initial;
            if (crcModel.CrcSize < 8)
            {
                crc <<= crcModel.BitOffsetTo8Bits;
            }

            foreach (byte b in bytes)
            {
                byte curByte = (crcModel.InputReflected ? CrcUtil.Reflect8(b) : b);

                /* update the MSB of crc value with next input byte */
                crc = (ulong)(crc ^ (ulong)((ulong)curByte << crcModel.ShiftByteLeftBits));
                /* this MSB byte value is the index into the lookup table */
                byte pos = (byte)(crc >> crcModel.ShiftByteLeftBits);
                /* shift out this index */
                crc = (ulong)(crc << 8);
                /* XOR-in remainder from lookup table using the calculated index */
                crc = (ulong)(crc ^ (ulong)crcTable[pos]);

                crc = crc & crcModel.FinalMask;
            }

            if (crcModel.CrcSize < 8)
            {
                crc >>= crcModel.BitOffsetTo8Bits;
            }

            crc = (crcModel.ResultReflected ? CrcUtil.ReflectGeneral(crc, crcModel.CrcSize) : crc);
            crc ^= crcModel.FinalXor;
            return crc;
        }
    }
}
