﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SunCRC.Crc.Crc1_64
{
    class Crc1_64Reflected
    {
        private Crc1_64Model crcModel;
        private ulong[] crcTable;

        public Crc1_64Reflected(Crc1_64Model model)
        {
            this.crcModel = Crc1_64Model.GetReflectedCrcModel(model);
            CalculateCrcTable();
        }

        public ulong Compute_Simple(byte[] bytes)
        {
            ulong crc = crcModel.Initial;

            ulong lowBitMask = 0x0000000000000001;

            foreach (byte b in bytes)
            {
                byte curByte = (crcModel.InputReflected ? CrcUtil.Reflect8(b) : b);
                crc ^= (ulong)curByte; /* move byte into LSB of 64bit CRC */

                for (int i = 0; i < 8; i++)
                {
                    if ((crc & lowBitMask) != 0)
                    {
                        crc = (ulong)((crc >> 1) ^ crcModel.Polynomial);
                    }
                    else
                    {
                        crc >>= 1;
                    }
                }

                crc = crc & crcModel.FinalMask;
            }

            crc = (crcModel.ResultReflected ? CrcUtil.ReflectGeneral(crc, crcModel.CrcSize) : crc);
            crc ^= crcModel.FinalXor;

            return crc;
        }

        /// <summary>
        /// Precalculate look up table.
        /// </summary>
        private void CalculateCrcTable()
        {
            crcTable = new ulong[256];
            ulong lowBitMask = 0x0000000000000001;

            for (int divident = 0; divident < 256; divident++)
            {
                ulong curByte = (ulong)divident;
                for (byte bit = 0; bit < 8; bit++)
                {
                    if ((curByte & lowBitMask) != 0)
                    {
                        curByte >>= 1;
                        curByte ^= crcModel.Polynomial;
                    }
                    else
                    {
                        curByte >>= 1;
                    }
                }

                crcTable[divident] = curByte & crcModel.FinalMask;
            }
        }

        /// <summary>
        /// Calculate using lookup table.
        /// </summary>
        /// <param name="bytes"></param>
        /// <returns></returns>
        public ulong Compute(byte[] bytes)
        {
            ulong crc = crcModel.Initial;

            foreach (byte b in bytes)
            {
                byte curByte = (crcModel.InputReflected ? CrcUtil.Reflect8(b) : b);

                /* update the LSB of crc value with next input byte */
                crc = (ulong)(crc ^ (ulong)(curByte));
                /* this byte value is the index into the lookup table */
                byte pos = (byte)(crc & 0xFF);
                /* shift out this index */
                crc = (ulong)(crc >> 8);
                /* XOR-in remainder from lookup table using the calculated index */
                crc = (ulong)(crc ^ (ulong)crcTable[pos]);

                crc = crc & crcModel.FinalMask;
            }

            crc = (crcModel.ResultReflected ? CrcUtil.ReflectGeneral(crc, crcModel.CrcSize) : crc);
            crc ^= crcModel.FinalXor;
            return crc;
        }
    }
}
