﻿using SunCRC.Crc.Crc1_64;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SunCRC.CrcTest.Crc1_64Test
{
    class Crc1_64Test
    {
        public class Crc1_64TestData
        {
            public Crc1_64Model CrcModel { get; private set; }
            public byte[] InputBytes { get; private set; }
            public ulong ResultCrc { get; private set; }

            public Crc1_64TestData(Crc1_64Model model, byte[] inputbytes, ulong result)
            {
                this.CrcModel = model;
                this.InputBytes = inputbytes;
                this.ResultCrc = result;
            }
        }

        private static Crc1_64TestData[] testdataEntries = new Crc1_64TestData[]
        {
            new Crc1_64TestData(Crc1_64Database.CRC3_GSM, CrcTestUtil.StandardTestDataInput, 0x4),
            new Crc1_64TestData(Crc1_64Database.CRC3_ROHC, CrcTestUtil.StandardTestDataInput, 0x6),
            new Crc1_64TestData(Crc1_64Database.CRC4_G704, CrcTestUtil.StandardTestDataInput, 0x7),
            new Crc1_64TestData(Crc1_64Database.CRC4_INTERLAKEN, CrcTestUtil.StandardTestDataInput, 0xb),
            new Crc1_64TestData(Crc1_64Database.CRC5_EPCC1G2, CrcTestUtil.StandardTestDataInput, 0x00),
            new Crc1_64TestData(Crc1_64Database.CRC5_G704, CrcTestUtil.StandardTestDataInput, 0x07),
            new Crc1_64TestData(Crc1_64Database.CRC5_USB, CrcTestUtil.StandardTestDataInput, 0x19),
            new Crc1_64TestData(Crc1_64Database.CRC6_CDMA2000A, CrcTestUtil.StandardTestDataInput, 0x0d),
            new Crc1_64TestData(Crc1_64Database.CRC6_CDMA2000B, CrcTestUtil.StandardTestDataInput, 0x3b),
            new Crc1_64TestData(Crc1_64Database.CRC6_DARC, CrcTestUtil.StandardTestDataInput, 0x26),
            new Crc1_64TestData(Crc1_64Database.CRC6_G704, CrcTestUtil.StandardTestDataInput, 0x06),
            new Crc1_64TestData(Crc1_64Database.CRC6_GSM, CrcTestUtil.StandardTestDataInput, 0x13),
            new Crc1_64TestData(Crc1_64Database.CRC7_MMC, CrcTestUtil.StandardTestDataInput, 0x75),
            new Crc1_64TestData(Crc1_64Database.CRC7_ROHC, CrcTestUtil.StandardTestDataInput, 0x53),
            new Crc1_64TestData(Crc1_64Database.CRC7_UMTS, CrcTestUtil.StandardTestDataInput, 0x61),
            new Crc1_64TestData(Crc1_64Database.CRC8_AUTOSAR, CrcTestUtil.StandardTestDataInput, 0xdf),
            new Crc1_64TestData(Crc1_64Database.CRC8_BLUETOOTH, CrcTestUtil.StandardTestDataInput, 0x26),
            new Crc1_64TestData(Crc1_64Database.CRC8_CDMA2000, CrcTestUtil.StandardTestDataInput, 0xda),
            new Crc1_64TestData(Crc1_64Database.CRC8_DARC, CrcTestUtil.StandardTestDataInput, 0x15),
            new Crc1_64TestData(Crc1_64Database.CRC8_DVBS2, CrcTestUtil.StandardTestDataInput, 0xbc),
            new Crc1_64TestData(Crc1_64Database.CRC8_GSMA, CrcTestUtil.StandardTestDataInput, 0x37),
            new Crc1_64TestData(Crc1_64Database.CRC8_GSMB, CrcTestUtil.StandardTestDataInput, 0x94),
            new Crc1_64TestData(Crc1_64Database.CRC8_HITAG, CrcTestUtil.StandardTestDataInput, 0xb4),
            new Crc1_64TestData(Crc1_64Database.CRC8_I4321, CrcTestUtil.StandardTestDataInput, 0xa1),
            new Crc1_64TestData(Crc1_64Database.CRC8_ICODE, CrcTestUtil.StandardTestDataInput, 0x7e),
            new Crc1_64TestData(Crc1_64Database.CRC8_LTE, CrcTestUtil.StandardTestDataInput, 0xea),
            new Crc1_64TestData(Crc1_64Database.CRC8_MAXIMDOW, CrcTestUtil.StandardTestDataInput, 0xa1),
            new Crc1_64TestData(Crc1_64Database.CRC8_MIFAREMAD, CrcTestUtil.StandardTestDataInput, 0x99),
            new Crc1_64TestData(Crc1_64Database.CRC8_NRSC5, CrcTestUtil.StandardTestDataInput, 0xf7),
            new Crc1_64TestData(Crc1_64Database.CRC8_OPENSAFETY, CrcTestUtil.StandardTestDataInput, 0x3e),
            new Crc1_64TestData(Crc1_64Database.CRC8_ROHC, CrcTestUtil.StandardTestDataInput, 0xd0),
            new Crc1_64TestData(Crc1_64Database.CRC8_SAEJ1850, CrcTestUtil.StandardTestDataInput, 0x4b),
            new Crc1_64TestData(Crc1_64Database.CRC8_SMBUS, CrcTestUtil.StandardTestDataInput, 0xf4),
            new Crc1_64TestData(Crc1_64Database.CRC8_TECH3250, CrcTestUtil.StandardTestDataInput, 0x97),
            new Crc1_64TestData(Crc1_64Database.CRC8_WCDMA, CrcTestUtil.StandardTestDataInput, 0x25),
            new Crc1_64TestData(Crc1_64Database.CRC10_ATM, CrcTestUtil.StandardTestDataInput, 0x199),
            new Crc1_64TestData(Crc1_64Database.CRC10_CDMA2000, CrcTestUtil.StandardTestDataInput, 0x233),
            new Crc1_64TestData(Crc1_64Database.CRC10_GSM, CrcTestUtil.StandardTestDataInput, 0x12a),
            new Crc1_64TestData(Crc1_64Database.CRC11_FLEXRAY, CrcTestUtil.StandardTestDataInput, 0x5a3),
            new Crc1_64TestData(Crc1_64Database.CRC11_UMTS, CrcTestUtil.StandardTestDataInput, 0x061),
            new Crc1_64TestData(Crc1_64Database.CRC12_CDMA2000, CrcTestUtil.StandardTestDataInput, 0xd4d),
            new Crc1_64TestData(Crc1_64Database.CRC12_DECT, CrcTestUtil.StandardTestDataInput, 0xf5b),
            new Crc1_64TestData(Crc1_64Database.CRC12_GSM, CrcTestUtil.StandardTestDataInput, 0xb34),
            new Crc1_64TestData(Crc1_64Database.CRC12_UMTS, CrcTestUtil.StandardTestDataInput, 0xdaf),
            new Crc1_64TestData(Crc1_64Database.CRC13_BBC, CrcTestUtil.StandardTestDataInput, 0x04fa),
            new Crc1_64TestData(Crc1_64Database.CRC14_DARC, CrcTestUtil.StandardTestDataInput, 0x082d),
            new Crc1_64TestData(Crc1_64Database.CRC14_GSM, CrcTestUtil.StandardTestDataInput, 0x30ae),
            new Crc1_64TestData(Crc1_64Database.CRC15_CAN, CrcTestUtil.StandardTestDataInput, 0x059e),
            new Crc1_64TestData(Crc1_64Database.CRC15_MPT1327, CrcTestUtil.StandardTestDataInput, 0x2566),
            new Crc1_64TestData(Crc1_64Database.CRC16_ARC, CrcTestUtil.StandardTestDataInput, 0xbb3d),
            new Crc1_64TestData(Crc1_64Database.CRC16_CDMA2000, CrcTestUtil.StandardTestDataInput, 0x4c06),
            new Crc1_64TestData(Crc1_64Database.CRC16_CMS, CrcTestUtil.StandardTestDataInput, 0xaee7),
            new Crc1_64TestData(Crc1_64Database.CRC16_DDS110, CrcTestUtil.StandardTestDataInput, 0x9ecf),
            new Crc1_64TestData(Crc1_64Database.CRC16_DECTR, CrcTestUtil.StandardTestDataInput, 0x007e),
            new Crc1_64TestData(Crc1_64Database.CRC16_DECTX, CrcTestUtil.StandardTestDataInput, 0x007f),
            new Crc1_64TestData(Crc1_64Database.CRC16_DNP, CrcTestUtil.StandardTestDataInput, 0xea82),
            new Crc1_64TestData(Crc1_64Database.CRC16_EN13757, CrcTestUtil.StandardTestDataInput, 0xc2b7),
            new Crc1_64TestData(Crc1_64Database.CRC16_GENIBUS, CrcTestUtil.StandardTestDataInput, 0xd64e),
            new Crc1_64TestData(Crc1_64Database.CRC16_GSM, CrcTestUtil.StandardTestDataInput, 0xce3c),
            new Crc1_64TestData(Crc1_64Database.CRC16_IBM3740, CrcTestUtil.StandardTestDataInput, 0x29b1),
            new Crc1_64TestData(Crc1_64Database.CRC16_IBMSDLC, CrcTestUtil.StandardTestDataInput, 0x906e),
            new Crc1_64TestData(Crc1_64Database.CRC16_ISOIEC144433A, CrcTestUtil.StandardTestDataInput, 0xbf05),
            new Crc1_64TestData(Crc1_64Database.CRC16_KERMIT, CrcTestUtil.StandardTestDataInput, 0x2189),
            new Crc1_64TestData(Crc1_64Database.CRC16_LJ1200, CrcTestUtil.StandardTestDataInput, 0xbdf4),
            new Crc1_64TestData(Crc1_64Database.CRC16_M17, CrcTestUtil.StandardTestDataInput, 0x772b),
            new Crc1_64TestData(Crc1_64Database.CRC16_MAXIMDOW, CrcTestUtil.StandardTestDataInput, 0x44c2),
            new Crc1_64TestData(Crc1_64Database.CRC16_MCRF4XX, CrcTestUtil.StandardTestDataInput, 0x6f91),
            new Crc1_64TestData(Crc1_64Database.CRC16_MODBUS, CrcTestUtil.StandardTestDataInput, 0x4b37),
            new Crc1_64TestData(Crc1_64Database.CRC16_NRSC5, CrcTestUtil.StandardTestDataInput, 0xa066),
            new Crc1_64TestData(Crc1_64Database.CRC16_OPENSAFETYA, CrcTestUtil.StandardTestDataInput, 0x5d38),
            new Crc1_64TestData(Crc1_64Database.CRC16_OPENSAFETYB, CrcTestUtil.StandardTestDataInput, 0x20fe),
            new Crc1_64TestData(Crc1_64Database.CRC16_PROFIBUS, CrcTestUtil.StandardTestDataInput, 0xa819),
            new Crc1_64TestData(Crc1_64Database.CRC16_RIELLO, CrcTestUtil.StandardTestDataInput, 0x63d0),
            new Crc1_64TestData(Crc1_64Database.CRC16_SPIFUJITSU, CrcTestUtil.StandardTestDataInput, 0xe5cc),
            new Crc1_64TestData(Crc1_64Database.CRC16_T10DIF, CrcTestUtil.StandardTestDataInput, 0xd0db),
            new Crc1_64TestData(Crc1_64Database.CRC16_TELEDISK, CrcTestUtil.StandardTestDataInput, 0x0fb3),
            new Crc1_64TestData(Crc1_64Database.CRC16_TMS37157, CrcTestUtil.StandardTestDataInput, 0x26b1),
            new Crc1_64TestData(Crc1_64Database.CRC16_UMTS, CrcTestUtil.StandardTestDataInput, 0xfee8),
            new Crc1_64TestData(Crc1_64Database.CRC16_USB, CrcTestUtil.StandardTestDataInput, 0xb4c8),
            new Crc1_64TestData(Crc1_64Database.CRC16_XMODEM, CrcTestUtil.StandardTestDataInput, 0x31c3),
            new Crc1_64TestData(Crc1_64Database.CRC17_CANFD, CrcTestUtil.StandardTestDataInput, 0x04f03),
            new Crc1_64TestData(Crc1_64Database.CRC21_CANFD, CrcTestUtil.StandardTestDataInput, 0x0ed841),
            new Crc1_64TestData(Crc1_64Database.CRC24_BLE, CrcTestUtil.StandardTestDataInput, 0xc25a56),
            new Crc1_64TestData(Crc1_64Database.CRC24_FLEXRAYA, CrcTestUtil.StandardTestDataInput, 0x7979bd),
            new Crc1_64TestData(Crc1_64Database.CRC24_FLEXRAYB, CrcTestUtil.StandardTestDataInput, 0x1f23b8),
            new Crc1_64TestData(Crc1_64Database.CRC24_INTERLAKEN, CrcTestUtil.StandardTestDataInput, 0xb4f3e6),
            new Crc1_64TestData(Crc1_64Database.CRC24_LTEA, CrcTestUtil.StandardTestDataInput, 0xcde703),
            new Crc1_64TestData(Crc1_64Database.CRC24_LTEB, CrcTestUtil.StandardTestDataInput, 0x23ef52),
            new Crc1_64TestData(Crc1_64Database.CRC24_OPENPGP, CrcTestUtil.StandardTestDataInput, 0x21cf02),
            new Crc1_64TestData(Crc1_64Database.CRC24_OS9, CrcTestUtil.StandardTestDataInput, 0x200fa5),
            new Crc1_64TestData(Crc1_64Database.CRC30_CDMA, CrcTestUtil.StandardTestDataInput, 0x04c34abf),
            new Crc1_64TestData(Crc1_64Database.CRC31_PHILIPS, CrcTestUtil.StandardTestDataInput, 0x0ce9e46c),
            new Crc1_64TestData(Crc1_64Database.CRC32_AIXM, CrcTestUtil.StandardTestDataInput, 0x3010bf7f),
            new Crc1_64TestData(Crc1_64Database.CRC32_AUTOSAR, CrcTestUtil.StandardTestDataInput, 0x1697d06a),
            new Crc1_64TestData(Crc1_64Database.CRC32_BASE91D, CrcTestUtil.StandardTestDataInput, 0x87315576),
            new Crc1_64TestData(Crc1_64Database.CRC32_BZIP2, CrcTestUtil.StandardTestDataInput, 0xfc891918),
            new Crc1_64TestData(Crc1_64Database.CRC32_CDROMEDC, CrcTestUtil.StandardTestDataInput, 0x6ec2edc4),
            new Crc1_64TestData(Crc1_64Database.CRC32_CKSUM, CrcTestUtil.StandardTestDataInput, 0x765e7680),
            new Crc1_64TestData(Crc1_64Database.CRC32_ISCSI, CrcTestUtil.StandardTestDataInput, 0xe3069283),
            new Crc1_64TestData(Crc1_64Database.CRC32_ISOHDLC, CrcTestUtil.StandardTestDataInput, 0xcbf43926),
            new Crc1_64TestData(Crc1_64Database.CRC32_JAMCRC, CrcTestUtil.StandardTestDataInput, 0x340bc6d9),
            new Crc1_64TestData(Crc1_64Database.CRC32_MEF, CrcTestUtil.StandardTestDataInput, 0xd2c22f51),
            new Crc1_64TestData(Crc1_64Database.CRC32_MPEG2, CrcTestUtil.StandardTestDataInput, 0x0376e6e7),
            new Crc1_64TestData(Crc1_64Database.CRC32_XFER, CrcTestUtil.StandardTestDataInput, 0xbd0be338),
            new Crc1_64TestData(Crc1_64Database.CRC40_GSM, CrcTestUtil.StandardTestDataInput, 0xd4164fc646),
            new Crc1_64TestData(Crc1_64Database.CRC64_ECMA182, CrcTestUtil.StandardTestDataInput, 0x6c40df5f0b497347),
            new Crc1_64TestData(Crc1_64Database.CRC64_GOISO, CrcTestUtil.StandardTestDataInput, 0xb90956c775a41001),
            new Crc1_64TestData(Crc1_64Database.CRC64_MS, CrcTestUtil.StandardTestDataInput, 0x75d4b74f024eceea),
            new Crc1_64TestData(Crc1_64Database.CRC64_REDIS, CrcTestUtil.StandardTestDataInput, 0xe9c6d914c4b8d9ca),
            new Crc1_64TestData(Crc1_64Database.CRC64_WE, CrcTestUtil.StandardTestDataInput, 0x62ec59e3f1a4f00a),
            new Crc1_64TestData(Crc1_64Database.CRC64_XZ, CrcTestUtil.StandardTestDataInput, 0x995dc9bbdf1939fa),

        };

        public void RunTests()
        {
            RunStandardCrcTests();
        }

        private void RunStandardCrcTests()
        {
            Console.Out.WriteLine("\nGeneral tests using Simple method:");
            foreach (Crc1_64TestData testEntry in testdataEntries)
            {
                Crc1_64 crcAlgo = new Crc1_64(testEntry.CrcModel);
                if (!CrcTestUtil.Assert64(crcAlgo.Compute_Simple(testEntry.InputBytes), testEntry.ResultCrc))
                {
                    Console.Out.WriteLine("Last failed CRC model: " + testEntry.CrcModel.Name);
                }
            }

            Console.Out.WriteLine("\nGeneral tests using Table method:");
            foreach (Crc1_64TestData testEntry in testdataEntries)
            {
                Crc1_64 crcAlgo = new Crc1_64(testEntry.CrcModel);
                if (!CrcTestUtil.Assert64(crcAlgo.Compute(testEntry.InputBytes), testEntry.ResultCrc))
                {
                    Console.Out.WriteLine("Last failed CRC model: " + testEntry.CrcModel.Name);
                }
            }

            // Reflected ones
            Console.Out.WriteLine("\nGeneral tests using Simple method (Reflected):");
            foreach (Crc1_64TestData testEntry in testdataEntries)
            {
                Crc1_64Reflected crcAlgo = new Crc1_64Reflected(testEntry.CrcModel);
                CrcTestUtil.Assert64(crcAlgo.Compute_Simple(testEntry.InputBytes), testEntry.ResultCrc);
            }

            Console.Out.WriteLine("\nGeneral tests using Table method (Reflected):");
            foreach (Crc1_64TestData testEntry in testdataEntries)
            {
                Crc1_64Reflected crcAlgo = new Crc1_64Reflected(testEntry.CrcModel);
                CrcTestUtil.Assert64(crcAlgo.Compute(testEntry.InputBytes), testEntry.ResultCrc);
            }
        }
    }
}
